using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Threading.Tasks;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Serialization;
using Ninject;
using TourDataManager.Entities;

namespace TourDataManager {
    public class GetEstatesUseCase {

        [Inject]
        public HttpClient httpClient{ get; set; }
        
        
        public struct BigantoErrorList {
            public List<BigantoError> Errors;
        }
        
        public struct BigantoError {
            public int Code;
            public string Message;

            public override string ToString(){ return $"code={Code};message={Message}"; }
        }

        public class BigantoErrorException : Exception {
            public BigantoErrorList ErrorsList;

            public BigantoErrorException(BigantoErrorList errors){
                ErrorsList = errors;
            }
        }

        /// <summary>
        /// <exception cref="HttpRequestException">Если сервер даст дубу</exception>
        /// <exception cref="BigantoErrorException">Если вместо массива эстейтов с сервера получены ошибки</exception>
        /// <exception cref="JsonException">Если вместо массива эстейтов или ошибки пришло нечто иное</exception>
        /// </summary>
        public async Task<Estate[]> GetEstates(){

            var response = await httpClient.GetAsync("https://biganto.com/api-novus/estates.getList?client=desktopplayer&client_version=3.0&v=2.0");             
            // throw HttpRequestException
            var content = await response.Content.ReadAsStringAsync();

            ErrorContext error = null;
            var errorHandlerSettings = new JsonSerializerSettings{
                Error = (sender,args) => {
                    error = args.ErrorContext;
                    args.ErrorContext.Handled = true;
                }
            };
            
            var estates = JsonConvert.DeserializeObject<Estate[]>(content,errorHandlerSettings);
            if (error == null) return estates;
            error = null;
            
            // Вместо массиво эстейтов в ответе может содержаться json ошибки
            // Предпринимается попытка десериализовать ошибки и выкинуть BigantoErrorException
            
            var exceptionList = JsonConvert.DeserializeObject<BigantoErrorList>(content,errorHandlerSettings);
            if (error == null) throw new BigantoErrorException(exceptionList);
            throw new JsonException("Something goes wrong",error.Error);
        }
    }
}